@@ -88,7 +88,19 @@ def test_chord_from_degrees(deg_in, semi_out):
     [
         ("", [0, 4, 7]),
         ("major", [0, 4, 7]),
+        ("inv1", [4, 7, 12]),
+        ("inv2", [7, 12, 16]),
+        ("inv3", [12, 16, 19]),
+        ("inv4", [16, 19, 24]),
+        ("inv5", [19, 24, 28]),
+        ("inv6", [24, 28, 31]),
+        ("inv7", [28, 31, 36]),
         ("minor", [0, 3, 7]),
+        ("sus2", [0, 2, 7]),
+        ("sus4", [0, 5, 7]),
+        ("7sus4", [0, 5, 7, 10]),
+        ("m7sus4b9no5", [0, 3, 5, 10, 13]),
+        ("augsus2", [0, 2, 8]),
         ("major7", [0, 4, 7, 11]),
         ("m7b5", [0, 3, 6, 10]),
         ("min7b5", [0, 3, 6, 10]),
@@ -96,12 +108,118 @@ def test_chord_from_degrees(deg_in, semi_out):
         ("o", [0, 3, 6, 9]),
         ("min", [0, 3, 7]),
         ("m", [0, 3, 7]),
+        ("13", [0, 4, 7, 10, 14, 17, 21]),
+        ("13no5no7b11#9", [0, 4, 10, 15, 16, 21]),
+        ("13b11#9no5no7", [0, 4, 15, 16, 21]),
+        ("13b11#9no5no7inv2", [12, 15, 16, 21]),
+        ("7#9", [0, 4, 7, 10, 15]),
+        ("7b9", [0, 4, 7, 10, 13]),
+        ("7b11", [0, 4, 7, 10, 14, 16]),
+        ("7#11", [0, 4, 7, 10, 14, 18]),
+        ("7b13", [0, 4, 7, 10, 14, 17, 20]),
+        ("7#13", [0, 4, 7, 10, 14, 17, 22]),
+        ("addb9", [0, 4, 7, 13]),
+        ("add9", [0, 4, 7, 14]),
+        ("add#9", [0, 4, 7, 15]),
+        ("addb11", [0, 4, 7, 16]),
+        ("add11", [0, 4, 7, 17]),
+        ("add#11", [0, 4, 7, 18]),
+        ("addb13", [0, 4, 7, 20]),
+        ("add13", [0, 4, 7, 21]),
+        ("add#13", [0, 4, 7, 22]),
     ],
 )
-def test_chord_from_degrees(name_in, semi_out):
+def test_chord_from_name_semitones(name_in, semi_out):
     assert Chord.from_name(name_in).semitones == semi_out
 
 
+@pytest.mark.parametrize(
+    "name_in, modifications",
+    [
+        ("", []),
+        ("major", []),
+        ("minor", []),
+        ("sus2", ["sus2"]),
+        ("sus4", ["sus4"]),
+        ("7sus4", ["sus4"]),
+        ("m7sus4b9no5", ["sus4", "b9", "no5"]),
+        ("augsus2", ["sus2"]),
+        ("major7", []),
+        ("m7b5", ["b5"]),
+        ("min7b5", ["b5"]),
+        ("ø", []),
+        ("o", []),
+        ("min", []),
+        ("m", []),
+        ("13", []),
+        ("13no5no7b11#9", ["no5", "no7", "b11", "#9"]),
+        ("13b11#9no5no7", ["b11", "#9", "no5", "no7"]),
+        ("7#9", ["#9"]),
+        ("7b9", ["b9"]),
+        ("7b11", ["b11"]),
+        ("7#11", ["#11"]),
+        ("7b13", ["b13"]),
+        ("7#13", ["#13"]),
+        ("addb9", ["addb9"]),
+        ("add9", ["add9"]),
+        ("add#9", ["add#9"]),
+        ("addb11", ["addb11"]),
+        ("add11", ["add11"]),
+        ("add#11", ["add#11"]),
+        ("addb13", ["addb13"]),
+        ("add13", ["add13"]),
+        ("add#13", ["add#13"]),
+    ],
+)
+def test_chord_from_name_modifications(name_in, modifications):
+    assert Chord.from_name(name_in).modifications == modifications
+
+
+# These could be improved
+@pytest.mark.parametrize(
+    "name_in, name_out",
+    [
+        ("", ""),
+        ("major", ""),
+        ("minor", "min"),
+        ("min", "min"),
+        ("m", "min"),
+        ("sus2", "sus2"),
+        ("sus4", "sus4"),
+        ("7sus4", "7sus4"),
+        ("m7sus4b9no5", Chord.UNNAMED),
+        ("augsus2", Chord.UNNAMED),
+        ("major7", "maj7"),
+        ("m7b5", "min7b5"),
+        ("min7b5", "min7b5"),
+        ("ø", "min7b5"),
+        ("o", "dim7"),
+        ("13", Chord.UNNAMED),
+        ("13no5no7b11#9", Chord.UNNAMED),
+        ("13b11#9no5no7", "maj7sus4(no5)/b6"),
+        ("7#9", Chord.UNNAMED),
+        ("7b9", Chord.UNNAMED),
+        ("7b11", Chord.UNNAMED),
+        ("7#11", Chord.UNNAMED),
+        ("7b13", Chord.UNNAMED),
+        ("7#13", Chord.UNNAMED),
+        ("addb9", "dim/7"),
+        ("add9", "min7(no5)/b6"),
+        ("add#9", "minmaj7(no5)/b6"),
+        ("addb11", ""),
+        ("add11", Chord.UNNAMED),
+        ("add#11", Chord.UNNAMED),
+        ("addb13", Chord.UNNAMED),
+        ("add13", Chord.UNNAMED),
+        ("add#13", "7"),
+    ],
+)
+def test_chord_name_roundtrip(name_in, name_out):
+    assert (
+        Chord.from_semitones(None, Chord.from_name(name_in).semitones).name == name_out
+    )
+
+
 @pytest.mark.parametrize(
     "name_in, int_out",
     [
@@ -118,7 +236,7 @@ def test_chord_from_degrees(name_in, semi_out):
         ("m", [3, 4]),
     ],
 )
-def test_chord_intervals(name_in, int_out):
+def test_chord_from_name_intervals(name_in, int_out):
     assert Chord.from_name(name_in).intervals() == int_out
 
 
@@ -147,7 +265,6 @@ def test_chord_repr(name_in, repr_out):
 def test_chord_add_root(name_in, octave):
     name_then_root = Chord.from_name(name_in).with_root(Note("A#", octave))
     name_and_root = ChordWithRoot.from_name("{}A#{}".format(octave, name_in))
-    print(name_and_root._keys())
     assert name_then_root == name_and_root
 
 
@@ -192,7 +309,7 @@ def test_chord_with_root(name_in, root_out, semi_out):
         ("Dbm", [3, 4]),
     ],
 )
-def test_chord_intervals(name_in, int_out):
+def test_chord_with_root_intervals(name_in, int_out):
     assert ChordWithRoot.from_name(name_in).intervals() == int_out
 
 
